module Result where

import Tree


data Result suc err
	= Success suc
	| Failure err
	deriving (Eq, Show)

onSuccess :: (a -> b) -> Result a err -> Result b err
onSuccess fun res = case res of
	Success s -> Success (fun s)
	Failure f -> Failure f


successes :: [Result a b] -> [a]
successes []     = []
successes (r:rs) = case r of
	Success s -> s : (successes rs)
	Failure f -> successes rs

failures :: [Result a b] -> [b]
failures []     = []
failures (r:rs) = case r of
	Success s -> failures rs
	Failure f -> f : (failures rs)



r_then :: (a -> Result b err) -> (b -> Result c err) -> (a -> Result c err)
r_then step1 step2 input = case step1 input of
	Failure err -> Failure err
	Success ans -> step2 ans


r_seq :: [a -> Result a err] -> (a -> Result a err)
r_seq funs arg = case funs of
	[] -> error "r_seq must be applied to a non-empty list."
	f:[] -> f arg
	f:fs -> case f arg of
		Success s -> r_seq fs s
		Failure f -> Failure f


r_alt :: [a -> Result a err] -> (a -> Result a err)
r_alt funs arg = case funs of
	[] -> error "r_alt must be applied to a non-empty list."
	f:[] -> f arg
	f:fs -> case f arg of
		Success s -> Success s
		Failure f -> r_alt fs arg


r_or :: (a -> Result a err) -> (a -> Result a err) -> (a -> Result a err)
r_or alt1 alt2 = r_alt [alt1, alt2]

summarize :: [Result a b] -> Result [a] [b]
summarize results = case failures results of
	[] -> Success (successes results)
	fs -> Failure fs


fSummarize :: Forest (Result a b) -> Result (Forest a) [b]
fSummarize forest = case failures (fFlatten forest) of
	[] -> Success (fMap (\(Success s) -> s) forest)
	fs -> Failure fs
